from typing import Any

from braintree.util.datetime_parser import parse_datetime as parse_datetime

binary_type = bytes

class Parser:
    doc: Any
    def __init__(self, xml) -> None: ...
    def parse(self): ...



from _typeshed import Incomplete
from typing import Any

from braintree.error_result import ErrorResult as ErrorResult
from braintree.exceptions.not_found_error import NotFoundError as NotFoundError
from braintree.paypal_account import PayPalAccount as PayPalAccount
from braintree.resource import Resource as Resource
from braintree.successful_result import SuccessfulResult as SuccessfulResult

class PayPalAccountGateway:
    gateway: Any
    config: Any
    def __init__(self, gateway) -> None: ...
    def find(self, paypal_account_token): ...
    def delete(self, paypal_account_token): ...
    def update(self, paypal_account_token, params: Incomplete | None = None): ...



from _typeshed import Incomplete

from braintree.configuration import Configuration as Configuration

class WebhookTesting:
    @staticmethod
    def sample_notification(kind, id, source_merchant_id: Incomplete | None = None): ...



from _typeshed import Incomplete
from typing import Any

from braintree.add_on import AddOn as AddOn
from braintree.address import Address as Address
from braintree.amex_express_checkout_card import AmexExpressCheckoutCard as AmexExpressCheckoutCard
from braintree.android_pay_card import AndroidPayCard as AndroidPayCard
from braintree.apple_pay_card import ApplePayCard as ApplePayCard
from braintree.authorization_adjustment import AuthorizationAdjustment as AuthorizationAdjustment
from braintree.configuration import Configuration as Configuration
from braintree.credit_card import CreditCard as CreditCard
from braintree.customer import Customer as Customer
from braintree.descriptor import Descriptor as Descriptor
from braintree.disbursement_detail import DisbursementDetail as DisbursementDetail
from braintree.discount import Discount as Discount
from braintree.dispute import Dispute as Dispute
from braintree.error_result import ErrorResult as ErrorResult
from braintree.europe_bank_account import EuropeBankAccount as EuropeBankAccount
from braintree.exceptions.not_found_error import NotFoundError as NotFoundError
from braintree.facilitated_details import FacilitatedDetails as FacilitatedDetails
from braintree.facilitator_details import FacilitatorDetails as FacilitatorDetails
from braintree.local_payment import LocalPayment as LocalPayment
from braintree.masterpass_card import MasterpassCard as MasterpassCard
from braintree.payment_instrument_type import PaymentInstrumentType as PaymentInstrumentType
from braintree.paypal_account import PayPalAccount as PayPalAccount
from braintree.paypal_here import PayPalHere as PayPalHere
from braintree.resource import Resource as Resource
from braintree.resource_collection import ResourceCollection as ResourceCollection
from braintree.risk_data import RiskData as RiskData
from braintree.samsung_pay_card import SamsungPayCard as SamsungPayCard
from braintree.status_event import StatusEvent as StatusEvent
from braintree.subscription_details import SubscriptionDetails as SubscriptionDetails
from braintree.successful_result import SuccessfulResult as SuccessfulResult
from braintree.three_d_secure_info import ThreeDSecureInfo as ThreeDSecureInfo
from braintree.transaction_line_item import TransactionLineItem as TransactionLineItem
from braintree.us_bank_account import UsBankAccount as UsBankAccount
from braintree.venmo_account import VenmoAccount as VenmoAccount
from braintree.visa_checkout_card import VisaCheckoutCard as VisaCheckoutCard

class Transaction(Resource):
    class CreatedUsing:
        FullInformation: str
        Token: str

    class GatewayRejectionReason:
        ApplicationIncomplete: str
        Avs: str
        AvsAndCvv: str
        Cvv: str
        Duplicate: str
        Fraud: str
        RiskThreshold: str
        ThreeDSecure: str
        TokenIssuance: str

    class Source:
        Api: str
        ControlPanel: str
        Recurring: str

    class EscrowStatus:
        HoldPending: str
        Held: str
        ReleasePending: str
        Released: str
        Refunded: str

    class Status:
        AuthorizationExpired: str
        Authorized: str
        Authorizing: str
        Failed: str
        GatewayRejected: str
        ProcessorDeclined: str
        Settled: str
        SettlementConfirmed: str
        SettlementDeclined: str
        SettlementFailed: str
        SettlementPending: str
        Settling: str
        SubmittedForSettlement: str
        Voided: str

    class Type:
        Credit: str
        Sale: str

    class IndustryType:
        Lodging: str
        TravelAndCruise: str
        TravelAndFlight: str

    class AdditionalCharge:
        Restaurant: str
        GiftShop: str
        MiniBar: str
        Telephone: str
        Laundry: str
        Other: str
    @staticmethod
    def adjust_authorization(transaction_id, amount): ...
    @staticmethod
    def clone_transaction(transaction_id, params): ...
    @staticmethod
    def cancel_release(transaction_id): ...
    @staticmethod
    def credit(params: Incomplete | None = None): ...
    @staticmethod
    def find(transaction_id): ...
    @staticmethod
    def hold_in_escrow(transaction_id): ...
    @staticmethod
    def refund(transaction_id, amount_or_options: Incomplete | None = None): ...
    @staticmethod
    def sale(params: Incomplete | None = None): ...
    @staticmethod
    def search(*query): ...
    @staticmethod
    def release_from_escrow(transaction_id): ...
    @staticmethod
    def submit_for_settlement(transaction_id, amount: Incomplete | None = None, params: Incomplete | None = None): ...
    @staticmethod
    def update_details(transaction_id, params: Incomplete | None = None): ...
    @staticmethod
    def void(transaction_id): ...
    @staticmethod
    def create(params): ...
    @staticmethod
    def clone_signature(): ...
    @staticmethod
    def create_signature(): ...
    @staticmethod
    def submit_for_settlement_signature(): ...
    @staticmethod
    def update_details_signature(): ...
    @staticmethod
    def refund_signature(): ...
    @staticmethod
    def submit_for_partial_settlement(transaction_id, amount, params: Incomplete | None = None): ...
    amount: Any
    tax_amount: Any
    discount_amount: Any
    shipping_amount: Any
    billing_details: Any
    credit_card_details: Any
    paypal_details: Any
    paypal_here_details: Any
    local_payment_details: Any
    europe_bank_account_details: Any
    us_bank_account: Any
    apple_pay_details: Any
    android_pay_card_details: Any
    amex_express_checkout_card_details: Any
    venmo_account_details: Any
    visa_checkout_card_details: Any
    masterpass_card_details: Any
    samsung_pay_card_details: Any
    sca_exemption_requested: Any
    customer_details: Any
    shipping_details: Any
    add_ons: Any
    discounts: Any
    status_history: Any
    subscription_details: Any
    descriptor: Any
    disbursement_details: Any
    disputes: Any
    authorization_adjustments: Any
    payment_instrument_type: Any
    risk_data: Any
    three_d_secure_info: Any
    facilitated_details: Any
    facilitator_details: Any
    network_transaction_id: Any
    def __init__(self, gateway, attributes) -> None: ...
    @property
    def vault_billing_address(self): ...
    @property
    def vault_credit_card(self): ...
    @property
    def vault_customer(self): ...
    @property
    def is_disbursed(self): ...
    @property
    def line_items(self): ...



from _typeshed import Incomplete

from braintree.attribute_getter import AttributeGetter as AttributeGetter

class RiskData(AttributeGetter):
    def __init__(self, attributes: Incomplete) -> None: ...



from typing import Any

from braintree.util.crypto import Crypto as Crypto

class SignatureService:
    private_key: Any
    hmac_hash: Any
    def __init__(self, private_key, hashfunc=...) -> None: ...
    def sign(self, data): ...
    def hash(self, data): ...



from _typeshed import Incomplete
from typing import Any

from braintree.customer import Customer as Customer
from braintree.error_result import ErrorResult as ErrorResult
from braintree.exceptions.not_found_error import NotFoundError as NotFoundError
from braintree.ids_search import IdsSearch as IdsSearch
from braintree.resource import Resource as Resource
from braintree.resource_collection import ResourceCollection as ResourceCollection
from braintree.successful_result import SuccessfulResult as SuccessfulResult

class CustomerGateway:
    gateway: Any
    config: Any
    def __init__(self, gateway) -> None: ...
    def all(self): ...
    def create(self, params: Incomplete | None = None): ...
    def delete(self, customer_id): ...
    def find(self, customer_id, association_filter_id: Incomplete | None = None): ...
    def search(self, *query): ...
    def update(self, customer_id, params: Incomplete | None = None): ...



from _typeshed import Incomplete
from typing import Any

from braintree.error_result import ErrorResult as ErrorResult
from braintree.exceptions.not_found_error import NotFoundError as NotFoundError
from braintree.resource import Resource as Resource
from braintree.resource_collection import ResourceCollection as ResourceCollection
from braintree.subscription import Subscription as Subscription
from braintree.successful_result import SuccessfulResult as SuccessfulResult
from braintree.transaction import Transaction as Transaction

class SubscriptionGateway:
    gateway: Any
    config: Any
    def __init__(self, gateway) -> None: ...
    def cancel(self, subscription_id): ...
    def create(self, params: Incomplete | None = None): ...
    def find(self, subscription_id): ...
    def retry_charge(self, subscription_id, amount: Incomplete | None = None, submit_for_settlement: bool = False): ...
    def search(self, *query): ...
    def update(self, subscription_id, params: Incomplete | None = None): ...



from typing import Any

from braintree.credit_card_verification import CreditCardVerification as CreditCardVerification
from braintree.credit_card_verification_search import CreditCardVerificationSearch as CreditCardVerificationSearch
from braintree.error_result import ErrorResult as ErrorResult
from braintree.exceptions.not_found_error import NotFoundError as NotFoundError
from braintree.ids_search import IdsSearch as IdsSearch
from braintree.resource_collection import ResourceCollection as ResourceCollection
from braintree.successful_result import SuccessfulResult as SuccessfulResult

class CreditCardVerificationGateway:
    gateway: Any
    config: Any
    def __init__(self, gateway) -> None: ...
    def find(self, verification_id): ...
    def search(self, *query): ...
    def create(self, params): ...



from _typeshed import Incomplete
from typing import Any

from braintree.configuration import Configuration as Configuration
from braintree.resource import Resource as Resource

class PayPalAccount(Resource):
    @staticmethod
    def find(paypal_account_token): ...
    @staticmethod
    def delete(paypal_account_token): ...
    @staticmethod
    def update(paypal_account_token, params: Incomplete | None = None): ...
    @staticmethod
    def signature(): ...
    subscriptions: Any
    def __init__(self, gateway, attributes) -> None: ...



from _typeshed import Incomplete

from braintree.configuration import Configuration as Configuration
from braintree.error_result import ErrorResult as ErrorResult
from braintree.resource import Resource as Resource
from braintree.successful_result import SuccessfulResult as SuccessfulResult

class Address(Resource):
    class ShippingMethod:
        SameDay: str
        NextDay: str
        Priority: str
        Ground: str
        Electronic: str
        ShipToStore: str
    @staticmethod
    def create(params: Incomplete | None = None): ...
    @staticmethod
    def delete(customer_id, address_id): ...
    @staticmethod
    def find(customer_id, address_id): ...
    @staticmethod
    def update(customer_id, address_id, params: Incomplete | None = None): ...
    @staticmethod
    def create_signature(): ...
    @staticmethod
    def update_signature(): ...



from _typeshed import Incomplete
from typing import Any

from braintree.credit_card import CreditCard as CreditCard
from braintree.error_result import ErrorResult as ErrorResult
from braintree.exceptions.not_found_error import NotFoundError as NotFoundError
from braintree.ids_search import IdsSearch as IdsSearch
from braintree.resource import Resource as Resource
from braintree.resource_collection import ResourceCollection as ResourceCollection
from braintree.successful_result import SuccessfulResult as SuccessfulResult

class CreditCardGateway:
    gateway: Any
    config: Any
    def __init__(self, gateway) -> None: ...
    def create(self, params: Incomplete | None = None): ...
    def delete(self, credit_card_token): ...
    def expired(self): ...
    def expiring_between(self, start_date, end_date): ...
    def find(self, credit_card_token): ...
    def forward(self, credit_card_token, receiving_merchant_id) -> None: ...
    def from_nonce(self, nonce): ...
    def update(self, credit_card_token, params: Incomplete | None = None): ...



from _typeshed import Incomplete
from typing import Any

from braintree import exceptions as exceptions
from braintree.client_token import ClientToken as ClientToken
from braintree.resource import Resource as Resource

class ClientTokenGateway:
    gateway: Any
    config: Any
    def __init__(self, gateway) -> None: ...
    def generate(self, params: Incomplete | None = None): ...



from braintree.resource import Resource as Resource

class ConnectedMerchantStatusTransitioned(Resource):
    def __init__(self, gateway, attributes) -> None: ...
    @property
    def merchant_id(self): ...



from typing import Any

from braintree.error_result import ErrorResult as ErrorResult
from braintree.exceptions.not_found_error import NotFoundError as NotFoundError
from braintree.merchant import Merchant as Merchant
from braintree.oauth_credentials import OAuthCredentials as OAuthCredentials
from braintree.resource import Resource as Resource
from braintree.resource_collection import ResourceCollection as ResourceCollection
from braintree.successful_result import SuccessfulResult as SuccessfulResult

class MerchantGateway:
    gateway: Any
    config: Any
    def __init__(self, gateway) -> None: ...
    def create(self, params): ...



from _typeshed import Incomplete

from braintree.address import Address as Address
from braintree.configuration import Configuration as Configuration
from braintree.resource import Resource as Resource

class PaymentMethod(Resource):
    @staticmethod
    def create(params: Incomplete | None = None): ...
    @staticmethod
    def find(payment_method_token): ...
    @staticmethod
    def update(payment_method_token, params): ...
    @staticmethod
    def delete(payment_method_token, options: Incomplete | None = None): ...
    @staticmethod
    def create_signature(): ...
    @staticmethod
    def signature(type): ...
    @staticmethod
    def update_signature(): ...
    @staticmethod
    def delete_signature(): ...



from _typeshed import Incomplete
from typing import Any

from braintree import version as version
from braintree.environment import Environment as Environment
from braintree.exceptions.authentication_error import AuthenticationError as AuthenticationError
from braintree.exceptions.authorization_error import AuthorizationError as AuthorizationError
from braintree.exceptions.gateway_timeout_error import GatewayTimeoutError as GatewayTimeoutError
from braintree.exceptions.http.connection_error import ConnectionError as ConnectionError
from braintree.exceptions.http.invalid_response_error import InvalidResponseError as InvalidResponseError
from braintree.exceptions.http.timeout_error import (
    ConnectTimeoutError as ConnectTimeoutError,
    ReadTimeoutError as ReadTimeoutError,
    TimeoutError as TimeoutError,
)
from braintree.exceptions.not_found_error import NotFoundError as NotFoundError
from braintree.exceptions.request_timeout_error import RequestTimeoutError as RequestTimeoutError
from braintree.exceptions.server_error import ServerError as ServerError
from braintree.exceptions.service_unavailable_error import ServiceUnavailableError as ServiceUnavailableError
from braintree.exceptions.too_many_requests_error import TooManyRequestsError as TooManyRequestsError
from braintree.exceptions.unexpected_error import UnexpectedError as UnexpectedError
from braintree.exceptions.upgrade_required_error import UpgradeRequiredError as UpgradeRequiredError
from braintree.util.xml_util import XmlUtil as XmlUtil

class Http:
    class ContentType:
        Xml: str
        Multipart: str
        Json: str
    @staticmethod
    def is_error_status(status): ...
    @staticmethod
    def raise_exception_from_status(status, message: Incomplete | None = None) -> None: ...
    config: Any
    environment: Any
    def __init__(self, config, environment: Incomplete | None = None) -> None: ...
    def post(self, path, params: Incomplete | None = None): ...
    def delete(self, path): ...
    def get(self, path): ...
    def put(self, path, params: Incomplete | None = None): ...
    def post_multipart(self, path, files, params: Incomplete | None = None): ...
    def http_do(self, http_verb, path, headers, request_body): ...
    def handle_exception(self, exception) -> None: ...



from typing import Any

class Search:
    class IsNodeBuilder:
        name: Any
        def __init__(self, name) -> None: ...
        def __eq__(self, value): ...
        def is_equal(self, value): ...

    class EqualityNodeBuilder(IsNodeBuilder):
        def __ne__(self, value): ...
        def is_not_equal(self, value): ...

    class KeyValueNodeBuilder:
        name: Any
        def __init__(self, name) -> None: ...
        def __eq__(self, value): ...
        def is_equal(self, value): ...
        def __ne__(self, value): ...
        def is_not_equal(self, value): ...

    class PartialMatchNodeBuilder(EqualityNodeBuilder):
        def starts_with(self, value): ...
        def ends_with(self, value): ...

    class EndsWithNodeBuilder:
        name: Any
        def __init__(self, name) -> None: ...
        def ends_with(self, value): ...

    class TextNodeBuilder(PartialMatchNodeBuilder):
        def contains(self, value): ...

    class Node:
        name: Any
        dict: Any
        def __init__(self, name, dict) -> None: ...
        def to_param(self): ...

    class MultipleValueNodeBuilder:
        name: Any
        whitelist: Any
        def __init__(self, name, whitelist=[]) -> None: ...
        def in_list(self, *values): ...
        def __eq__(self, value): ...

    class MultipleValueOrTextNodeBuilder(TextNodeBuilder, MultipleValueNodeBuilder):
        def __init__(self, name, whitelist=[]) -> None: ...

    class RangeNodeBuilder:
        name: Any
        def __init__(self, name) -> None: ...
        def __eq__(self, value): ...
        def is_equal(self, value): ...
        def __ge__(self, min): ...
        def greater_than_or_equal_to(self, min): ...
        def __le__(self, max): ...
        def less_than_or_equal_to(self, max): ...
        def between(self, min, max): ...



from typing import Any

from braintree.bin_data import BinData as BinData
from braintree.configuration import Configuration as Configuration
from braintree.resource import Resource as Resource
from braintree.three_d_secure_info import ThreeDSecureInfo as ThreeDSecureInfo

class PaymentMethodNonce(Resource):
    @staticmethod
    def create(payment_method_token, params={}): ...
    @staticmethod
    def find(payment_method_nonce): ...
    three_d_secure_info: Any
    authentication_insight: Any
    bin_data: Any
    def __init__(self, gateway, attributes) -> None: ...



from typing import Any

from braintree.credit_card import CreditCard as CreditCard
from braintree.search import Search as Search
from braintree.transaction import Transaction as Transaction
from braintree.util import Constants as Constants

class TransactionSearch:
    billing_first_name: Any
    billing_company: Any
    billing_country_name: Any
    billing_extended_address: Any
    billing_last_name: Any
    billing_locality: Any
    billing_postal_code: Any
    billing_region: Any
    billing_street_address: Any
    credit_card_cardholder_name: Any
    currency: Any
    customer_company: Any
    customer_email: Any
    customer_fax: Any
    customer_first_name: Any
    customer_id: Any
    customer_last_name: Any
    customer_phone: Any
    customer_website: Any
    id: Any
    order_id: Any
    payment_method_token: Any
    processor_authorization_code: Any
    europe_bank_account_iban: Any
    settlement_batch_id: Any
    shipping_company: Any
    shipping_country_name: Any
    shipping_extended_address: Any
    shipping_first_name: Any
    shipping_last_name: Any
    shipping_locality: Any
    shipping_postal_code: Any
    shipping_region: Any
    shipping_street_address: Any
    paypal_payer_email: Any
    paypal_payment_id: Any
    paypal_authorization_id: Any
    credit_card_unique_identifier: Any
    store_id: Any
    credit_card_expiration_date: Any
    credit_card_number: Any
    user: Any
    ids: Any
    merchant_account_id: Any
    payment_instrument_type: Any
    store_ids: Any
    created_using: Any
    credit_card_card_type: Any
    credit_card_customer_location: Any
    source: Any
    status: Any
    type: Any
    refund: Any
    amount: Any
    authorization_expired_at: Any
    authorized_at: Any
    created_at: Any
    disbursement_date: Any
    dispute_date: Any
    failed_at: Any
    gateway_rejected_at: Any
    processor_declined_at: Any
    settled_at: Any
    submitted_for_settlement_at: Any
    voided_at: Any



from typing import Any

from braintree import Subscription as Subscription
from braintree.search import Search as Search
from braintree.util import Constants as Constants

class SubscriptionSearch:
    billing_cycles_remaining: Any
    created_at: Any
    days_past_due: Any
    id: Any
    ids: Any
    in_trial_period: Any
    merchant_account_id: Any
    next_billing_date: Any
    plan_id: Any
    price: Any
    status: Any
    transaction_id: Any
